unit uMain;
                 
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, ImgList, dOPC;
       
type
  TForm1 = class(TForm)
    TopPanel: TPanel;
    TreeView1: TTreeView;
    BottomPanel: TPanel;
    Splitter1: TSplitter;
    ServerCombo: TComboBox;
    Label1: TLabel;
    bConnect: TButton;
    bDisconnect: TButton;
    ImageList1: TImageList;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    LServer: TLabel;
    LStartTime: TLabel;
    LState: TLabel;
    LInfo: TLabel;
    ListBox1: TListBox;
    Label9: TLabel;
    LError: TLabel;
    OPCServer: TdOPCServer;
    procedure ServerComboDropDown(Sender: TObject);
    procedure bConnectClick(Sender: TObject);
    procedure bDisconnectClick(Sender: TObject);
    procedure TreeView1Change(Sender: TObject; Node: TTreeNode);
    procedure FormShow(Sender: TObject);
    procedure OPCServerConnect(Sender: TObject);
    procedure OPCServerDisconnect(Sender: TObject);
    procedure OPCServerError(Sender: TObject; Errorstring: String;
      var RaiseException: Boolean);
  private
    ErrorCounter : integer;
    procedure ShowLeafsInListBox;
    procedure InfoClear;
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

// fills a Treeview with all Branches of the OPC Server
//@params Browser  a created TdvOPCBrowser object
//@params Tree     Treeview to fill
//@params Node     call always with nil
procedure BranchesToTreeView(Browser: TdOPCBrowser; Tree: TTreeView; Node : TTreeNode = nil);
var
  i       : integer;
  S       : TStringList;
  NewNode : TTreeNode;
begin
  Browser.ShowBranches;            // get all Branches in this level from OPC Server
  S := TStringList.Create;
  S.Assign(Browser.Items);         // save Items in new StringList
  for i := 0 to S.Count-1 do       // for all Items in stringlist
  begin
      NewNode := Tree.Items.AddChild(Node,S[i]);
      if assigned(Tree.Images) then                // only for selected images
        if Tree.Images.Count > 1 then
          NewNode.SelectedIndex := 1;
      if Browser.MoveDown(S[i]) then               // one Level down
      begin
        BranchesToTreeView(Browser,Tree,NewNode);  // rekusive call
        Browser.MoveUp;                            // back to old Level
      end;
  end;
  S.Free;
end;

// get complete Path delimited with <Delimiter> for the selected Node
// Test.Test.Test
function GetTreeViewPathStr(Tree: TTreeview; Delimiter: char): string;
var
  node    : TTreeNode;
  PathStr : string;
  D       : string;
begin
  D := Delimiter;
  Node := Tree.Selected;
  PathStr := '';
  if Node <> nil then
  begin
    repeat
      PathStr :=  '"'+Node.Text+'"'+ D + PathStr ;
      Node := Node.Parent;
    until Node = nil;
    delete(PathStr,length(PathStr),1);  // delete last delimiter
  end;
  result := PathStr;
end;


procedure TForm1.ServerComboDropDown(Sender: TObject);
begin
  if trim(ServerCombo.Items.Text) = '' then  // if no OPC Servernames loaded
  begin
    Screen.Cursor := crHourGlass;
    GetOPCServers(ServerCombo.Items);        // get Servernames from registry
    Screen.Cursor := crDefault;
  end;
end;

procedure TForm1.bConnectClick(Sender: TObject);
begin
   OpcServer.Active     := false;              // disconnect from old server
   OpcServer.ServerName := ServerCombo.Text;   // set Servername from Combobox
   OpcServer.Active     := true;               // connect to Server
end;

procedure TForm1.bDisconnectClick(Sender: TObject);
begin
  OpcServer.Active     := false;              // disconnect from old server
end;

procedure TForm1.InfoClear;
begin
  ErrorCounter := 0;
  LServer.Caption := '';
  LState.Caption := '';
  LInfo.Caption := '';
  LStartTime.Caption := '';
  LError.Caption := '';
end;

procedure TForm1.ShowLeafsInListBox;
var
  PathList  : TStrings;
  i         : integer;
begin
  PathList := TStringlist.Create;                          // create new Stringlist
  PathList.Commatext := GetTreeViewPathStr(TreeView1,','); //
  OpcServer.Browser.MovetoRoot;                            // Move to Top
  for i := 0 to PathList.Count-1 do                        // now Move down step by step
    OpcServer.Browser.MoveDown(PathList[i]);
  OpcServer.Browser.ShowLeafs;                             // get all Items in Path
  ListBox1.Items.Assign(OpcServer.Browser.Items);          // Show in Listbox
  PathList.Free;
end;

procedure TForm1.TreeView1Change(Sender: TObject; Node: TTreeNode);
begin
  if OPCServer.Active then    // only if connected to opc server
    ShowLeafsInListBox;    // if Tree change then show leafs in Path
end;

procedure TForm1.FormShow(Sender: TObject);
begin
   InfoClear;
end;

procedure TForm1.OPCServerConnect(Sender: TObject);
begin
  // show info in Statusbar
  LServer.Caption := OpcServer.Servername;
  LState.Caption  := OpcServer.ServerStateStr;
  LInfo.Caption   := OpcServer.VendorInfo;
  LStartTime.Caption := DateTimetoStr(OpcServer.StartTime);

  BranchesToTreeView(OpcServer.Browser,TreeView1); // get all Pathes from OPC Server
  ShowLeafsInListBox;                           // Show Items in Path
end;

procedure TForm1.OPCServerDisconnect(Sender: TObject);
begin
  TreeView1.Items.Clear; // clear all lists
  ListBox1.Items.Clear;
  InfoClear;
end;

procedure TForm1.OPCServerError(Sender: TObject; Errorstring: String;
  var RaiseException: Boolean);
begin
  inc(ErrorCounter);
  LError.Caption := InttoStr(ErrorCounter) + ' '+ErrorString;
end;

end.
